@@ -675,11 +675,17 @@ from_string_func(JSContext *context,
         char *encoded;
         gsize bytes_written;
         GError *error;
-        jschar *u16_chars;
+        const jschar *u16_chars;
         gsize u16_len;
 
+#ifdef HAVE_JS_GETSTRINGCHARS
         u16_chars = JS_GetStringChars(JSVAL_TO_STRING(argv[0]));
         u16_len = JS_GetStringLength(JSVAL_TO_STRING(argv[0]));
+#else
+        u16_chars = JS_GetStringCharsAndLength(context, JSVAL_TO_STRING(argv[0]), &u16_len);
+        if (u16_chars == NULL)
+            goto out;
+#endif
 
         error = NULL;
         encoded = g_convert((char*) u16_chars,
