@@ -254,8 +254,12 @@ gboolean
 gjs_init_context_standard (JSContext       *context)
 {
     JSObject *global;
-#ifdef HAVE_JS_NEWGLOBALOBJECT
+#ifdef HAVE_MOZJS_2
+#ifdef HAVE_JS_NEWCOMPARTMENTANDGLOBALOBJECT
+    global = JS_NewCompartmentAndGlobalObject(context, &global_class, NULL);
+#else
     global = JS_NewGlobalObject(context, &global_class);
+#endif /* HAVE_JS_NEWCOMPARTMENTANDGLOBALOBJECT */
     if (global == NULL)
         return FALSE;
 #else
@@ -263,7 +267,7 @@ gjs_init_context_standard (JSContext       *context)
     if (global == NULL)
         return FALSE;
     JS_SetGlobalObject(context, global);
-#endif
+#endif /* HAVE_MOZJS_2 */
     if (!JS_InitStandardClasses(context, global))
         return FALSE;
     return TRUE;
@@ -535,6 +539,8 @@ gjs_init_class_dynamic(JSContext      *context,
                                  constructor, nargs,
                                  ps, fs,
                                  static_ps, static_fs);
+        if (prototype == NULL)
+            goto error;
 
         /* Retrieve the property again so we can define it in
          * in_object
@@ -564,19 +570,11 @@ gjs_init_class_dynamic(JSContext      *context,
     return NULL;
 }
 
-gboolean
-gjs_check_constructing(JSContext *context)
+void
+gjs_throw_constructor_error(JSContext *context)
 {
-    JS_BeginRequest(context);
-    if (!JS_IsConstructing(context)) {
-        JS_EndRequest(context);
-        gjs_throw(context,
-                  "Constructor called as normal method. Use 'new SomeObject()' not 'SomeObject()'");
-        return FALSE;
-    }
-
-    JS_EndRequest(context);
-    return TRUE;
+    gjs_throw(context,
+              "Constructor called as normal method. Use 'new SomeObject()' not 'SomeObject()'");
 }
 
 void*
@@ -761,12 +759,20 @@ gjs_define_string_array(JSContext   *context,
     return array;
 }
 
-const char*
+/**
+ * gjs_value_debug_string:
+ * @context:
+ * @value: Any JavaScript value
+ *
+ * Returns: A UTF-8 encoded string describing @value
+ */
+char*
 gjs_value_debug_string(JSContext      *context,
                        jsval           value)
 {
     JSString *str;
-    const char *bytes;
+    char *bytes;
+    char *debugstr;
 
     JS_BeginRequest(context);
 
@@ -784,24 +790,35 @@ gjs_value_debug_string(JSContext      *context,
                 str = JS_NewStringCopyZ(context, klass->name);
                 JS_ClearPendingException(context);
                 if (str == NULL) {
-                    return "[out of memory copying class name]";
+                    return g_strdup("[out of memory copying class name]");
                 }
             } else {
                 gjs_log_exception(context, NULL);
-                return "[unknown object]";
+                return g_strdup("[unknown object]");
             }
         } else {
-            return "[unknown non-object]";
+            return g_strdup("[unknown non-object]");
         }
     }
 
     g_assert(str != NULL);
 
-    bytes = JS_GetStringBytes(str);
-
+#ifdef HAVE_JS_GETSTRINGBYTES
+    bytes = g_strdup(JS_GetStringBytes(str));
+#else
+    size_t len = JS_GetStringEncodingLength(context, str);
+    if (len != (size_t)(-1)) {
+        bytes = g_malloc((len + 1) * sizeof(char));
+        JS_EncodeStringToBuffer(str, bytes, len);
+        bytes[len] = '\0';
+    }
+#endif
     JS_EndRequest(context);
 
-    return bytes;
+    debugstr = _gjs_g_utf8_make_valid(bytes);
+    g_free(bytes);
+
+    return debugstr;
 }
 
 void
@@ -834,7 +851,8 @@ gjs_log_object_props(JSContext      *context,
 
     while (!JSID_IS_VOID(prop_id)) {
         jsval propval;
-        const char *name;
+        char *name;
+        char *debugstr;
 
         if (!gjs_get_string_id(context, prop_id, &name))
             goto next;
@@ -842,12 +860,15 @@ gjs_log_object_props(JSContext      *context,
         if (!gjs_object_get_property(context, obj, name, &propval))
             goto next;
 
+        debugstr = gjs_value_debug_string(context, propval);
         gjs_debug(topic,
                   "%s%s = '%s'",
                   prefix, name,
-                  gjs_value_debug_string(context, propval));
+                  debugstr);
+        g_free(debugstr);
 
     next:
+        g_free(name);
         prop_id = JSID_VOID;
         if (!JS_NextProperty(context, props_iter, &prop_id))
             break;
@@ -865,6 +886,7 @@ gjs_explain_scope(JSContext  *context,
     JSObject *global;
     JSObject *parent;
     GString *chain;
+    char *debugstr;
 
     gjs_debug(GJS_DEBUG_SCOPE,
               "=== %s ===",
@@ -880,14 +902,16 @@ gjs_explain_scope(JSContext  *context,
               "");
 
     global = JS_GetGlobalObject(context);
+    debugstr = gjs_value_debug_string(context, OBJECT_TO_JSVAL(global));
     gjs_debug(GJS_DEBUG_SCOPE,
               "  Global: %p %s",
-              global, gjs_value_debug_string(context, OBJECT_TO_JSVAL(global)));
+              global, debugstr);
+    g_free(debugstr);
 
     parent = JS_GetScopeChain(context);
     chain = g_string_new(NULL);
     while (parent != NULL) {
-        const char *debug;
+        char *debug;
         debug = gjs_value_debug_string(context, OBJECT_TO_JSVAL(parent));
 
         if (chain->len > 0)
@@ -895,6 +919,7 @@ gjs_explain_scope(JSContext  *context,
 
         g_string_append_printf(chain, "%p %s",
                                parent, debug);
+        g_free(debug);
         parent = JS_GetParent(context, parent);
     }
     gjs_debug(GJS_DEBUG_SCOPE,
@@ -1132,12 +1157,13 @@ log_prop(JSContext  *context,
          const char *what)
 {
     if (JSVAL_IS_STRING(id)) {
-        const char *name;
+        char *name;
 
-        name = gjs_string_get_ascii(id);
+        name = gjs_string_get_ascii(context, id);
         gjs_debug(GJS_DEBUG_PROPS,
                   "prop %s: %s",
                   name, what);
+        g_free(name);
     } else if (JSVAL_IS_INT(id)) {
         gjs_debug(GJS_DEBUG_PROPS,
                   "prop %d: %s",
@@ -1247,6 +1273,52 @@ gjs_date_from_time_t (JSContext *context, time_t time)
     return result;
 }
 
+/**
+ * gjs_value_to_int64:
+ * @context: the Javascript context object
+ * @val: Javascript value to convert
+ * @gint64: location to store the return value
+ *
+ * Converts a Javascript value into the nearest 64 bit signed value.
+ *
+ * This function behaves indentically for rounding to JSValToInt32(), which
+ * means that it rounds (0.5 toward positive infinity) rather than doing
+ * a C-style truncation to 0. If we change to using JSValToEcmaInt32() then
+ * this should be changed to match.
+ *
+ * Return value: If the javascript value converted to a number (see
+ *   JS_ValueToNumber()) is NaN, or outside the range of 64-bit signed
+ *   numbers, fails and sets an exception. Otherwise returns the value
+ *   rounded to the nearest 64-bit integer. Like JS_ValueToInt32(),
+ *   undefined throws, but null => 0, false => 0, true => 1.
+ */
+JSBool
+gjs_value_to_int64  (JSContext  *context,
+                     const jsval val,
+                     gint64     *result)
+{
+    if (JSVAL_IS_INT (val)) {
+        *result = JSVAL_TO_INT (val);
+        return JS_TRUE;
+    } else {
+        double value_double;
+        if (!JS_ValueToNumber(context, val, &value_double))
+            return JS_FALSE;
+
+        if (isnan(value_double) ||
+            value_double < G_MININT64 ||
+            value_double > G_MAXINT64) {
+
+            gjs_throw(context,
+                      "Value is not a valid 64-bit integer");
+            return JS_FALSE;
+        }
+
+        *result = (gint64)(value_double + 0.5);
+        return JS_TRUE;
+    }
+}
+
 /**
  * gjs_parse_args:
  * @context:
